package com.jiuqi.mde;

import java.util.Date;
import java.io.IOException;
import java.io.Writer;
import java.io.File;
import java.text.SimpleDateFormat;
import java.io.BufferedWriter;
import java.io.FileWriter;

public class MDESqlLogger
{
    private String _LogFilePath;
    private boolean _LogStackTrace;
    private boolean _LogInsert;
    private boolean _LogDelete;
    private boolean _LogUpdate;
    private int _CurrentSize;
    private FileWriter _FileWriter;
    private BufferedWriter _BufferedWriter;
    private static SimpleDateFormat TIME_FORMAT;
    private static SimpleDateFormat FILE_FORMAT;
    public static MDESqlLogger Sql_Logger;
    
    static {
        MDESqlLogger.TIME_FORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss:SSS");
        MDESqlLogger.FILE_FORMAT = new SimpleDateFormat("MMdd_HHmmss_SSS");
        MDESqlLogger.Sql_Logger = null;
    }
    
    public MDESqlLogger(final String logFilePath, final boolean logStackTrace, final boolean logInsert, final boolean logDelete, final boolean logUpdate) throws IOException {
        this._LogStackTrace = false;
        this._LogInsert = true;
        this._LogDelete = true;
        this._LogUpdate = true;
        this._LogFilePath = logFilePath;
        this._LogStackTrace = logStackTrace;
        this._LogInsert = logInsert;
        this._LogDelete = logDelete;
        this._LogUpdate = logUpdate;
        if (!this._LogFilePath.endsWith(File.separator)) {
            this._LogFilePath = String.valueOf(this._LogFilePath) + File.separator;
        }
        final File directory = new File(this._LogFilePath);
        if (!directory.isDirectory()) {
            directory.mkdirs();
        }
        this._FileWriter = new FileWriter(this.getNextFile(), false);
        this._BufferedWriter = new BufferedWriter(this._FileWriter, 1048576);
    }
    
    public static String getLogTime() {
        return MDESqlLogger.TIME_FORMAT.format(new Date());
    }
    
    public static String getStackTrace() {
        final Exception e = new Exception("");
        final StackTraceElement[] traces = e.getStackTrace();
        String result = "\"";
        for (int i = 0; i < traces.length; ++i) {
            final StackTraceElement trace = traces[i];
            final String className = trace.getClassName();
            if (className.startsWith("jsp_servlet") || (className.startsWith("com.jiuqi") && !className.startsWith("com.jiuqi.mde"))) {
                result = String.valueOf(result) + trace.toString() + "\n";
            }
        }
        return String.valueOf(result) + "\"\n\r";
    }
    
    private String getNextFile() {
        return String.valueOf(this._LogFilePath) + MDESqlLogger.FILE_FORMAT.format(new Date()) + ".log";
    }
    
    public void LogInsertSql(String sqlInfo) {
        if (this._LogInsert) {
            if (this._LogStackTrace) {
                sqlInfo = String.valueOf(sqlInfo) + getStackTrace();
            }
            this.LogSqlInfo(sqlInfo);
        }
    }
    
    public void LogDeleteSql(String sqlInfo) {
        if (this._LogDelete) {
            if (this._LogStackTrace) {
                sqlInfo = String.valueOf(sqlInfo) + getStackTrace();
            }
            this.LogSqlInfo(sqlInfo);
        }
    }
    
    public void LogUpdateSql(String sqlInfo) {
        if (this._LogUpdate) {
            if (this._LogStackTrace) {
                sqlInfo = String.valueOf(sqlInfo) + getStackTrace();
            }
            this.LogSqlInfo(sqlInfo);
        }
    }
    
    private synchronized void LogSqlInfo(final String sqlInfo) {
        try {
            this._BufferedWriter.write(sqlInfo);
            this._CurrentSize += sqlInfo.length();
            if (this._CurrentSize > 16777216) {
                this._BufferedWriter.close();
                this._FileWriter.close();
                this._CurrentSize = 0;
                this._FileWriter = new FileWriter(this.getNextFile(), false);
                this._BufferedWriter = new BufferedWriter(this._FileWriter, 1048576);
            }
        }
        catch (final IOException ex) {
            ex.printStackTrace();
        }
    }
    
    public synchronized void close() throws IOException {
        this._BufferedWriter.close();
        this._FileWriter.close();
    }
    
    public static void startSQLLog(final String logFilePath, final boolean logStackTrace, final boolean logInsert, final boolean logDelete, final boolean logUpdate) throws IOException {
        if (MDESqlLogger.Sql_Logger != null) {
            throw new IOException("跟踪已经开始!");
        }
        MDESqlLogger.Sql_Logger = new MDESqlLogger(logFilePath, logStackTrace, logInsert, logDelete, logUpdate);
    }
    
    public static void stopSQLLog() throws IOException {
        if (MDESqlLogger.Sql_Logger == null) {
            throw new IOException("跟踪还没有开始!");
        }
        final MDESqlLogger temp = MDESqlLogger.Sql_Logger;
        MDESqlLogger.Sql_Logger = null;
        temp.close();
    }
}
